-----------------------------------------------------------------------------------------------
-- Client Lua Script for InviteFilter
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"

-----------------------------------------------------------------------------------------------
-- InviteFilter Globals
-----------------------------------------------------------------------------------------------
local strCurrentInvitor = nil
local strCurrentGuild = nil
local strCurrentGuildType = nil

-----------------------------------------------------------------------------------------------
-- InviteFilter Module Definition
-----------------------------------------------------------------------------------------------
local InviteFilter = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
local kstrWndArenaTeamInvite = "wndArenaTeamInvite"
local kstrParentAddon = "ArenaTeam"

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function InviteFilter:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    o.data = {}
	o.data.tFilteredList = {}
	o.data.tAlwaysAllowList = {}
	o.data.bHandleCirles = false
	o.data.bHandleArenaTeams = true
	o.data.bHandleGuilds = false
	o.data.bHandleWarParties = false

    return o
end

function InviteFilter:Init()
	local bHasConfigureFunction = true
	local strConfigureButtonText = "InviteFilter"
	local tDependencies = {
		"ArenaTeam"
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- InviteFilter OnLoad
-----------------------------------------------------------------------------------------------
function InviteFilter:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("InviteFilter.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- InviteFilter OnDocLoaded
-----------------------------------------------------------------------------------------------
function InviteFilter:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "InviteFilterForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
		self.arenaTeamAddon = Apollo.GetAddon(kstrParentAddon)
		self.wndMain:Show(false, true)

		Apollo.RegisterSlashCommand("invitefilter", "OnSlashCommand", self)
		Apollo.RegisterEventHandler("GuildInvite", "OnGuildInvite", self)  -- notification you got a guild/circle invite
	end
end

function InviteFilter:OnDependencyError(strDep, strError)
	self:DisplayAddonMessage("ArenaTeam addon not loaded!")
	return false
end


-----------------------------------------------------------------------------------------------
-- InviteFilter OnSave/OnRestore/OnConfigure
-----------------------------------------------------------------------------------------------
function InviteFilter:OnSave(eLevel) 
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end 
	
	return self.data
end


function InviteFilter:OnRestore(eLevel, tData)
	self.data.tFilteredList = self:SetValueOrDefault(tData.tFilteredList, self.data.tFilteredList)
	self.data.tAlwaysAllowList = self:SetValueOrDefault(tData.tAlwaysAllowList, self.data.tAlwaysAllowList)
	self.data.bHandleCirles = self:SetValueOrDefault(tData.bHandleCirles, self.data.bHandleCirles)
	self.data.bHandleArenaTeams = self:SetValueOrDefault(tData.bHandleArenaTeams, self.data.bHandleArenaTeams)
	self.data.bHandleGuilds = self:SetValueOrDefault(tData.bHandleGuilds, self.data.bHandleGuilds)
	self.data.bHandleWarParties = self:SetValueOrDefault(tData.bHandleWarParties, self.data.bHandleWarParties)
end

function InviteFilter:OnConfigure()
	self:InitAndShowOptions()
end

-----------------------------------------------------------------------------------------------
-- InviteFilter Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here
function InviteFilter:OnSlashCommand(cmd, strArgs)
	if strArgs:lower() == "" or strArgs:lower() == "options" then
		self:OnConfigure()
	else
		local args = {}
		for arg in strArgs:gmatch("%S+") do 
			table.insert(args, arg) 
		end
		
		local argCmd = args[1]:lower()
		if not args[1] or (argCmd ~= "filter" and argCmd ~= "allow" and argCmd ~= "query" and argCmd ~= "remove") then
			self:DisplayAddonMessage(string.format("Unknown command - %s", argCmd))
			return
		end
		
		if not args[2] then
			self:DisplayAddonMessage("2nd parameter expected")
			return
		end
		
		if argCmd == "filter" then
			self:AddToFilterList(args[2])
		elseif argCmd == "allow" then
			self:AddToAllowList(args[2])
		elseif argCmd == "query" then
			self:QueryLists(args[2])
		elseif argCmd == "remove" then
			self:RemoveFromLists(args[2]) 
		end
	end
end

function InviteFilter:AddToFilterList(strValue)
	--remove from allow list if exists
	local nIndex = self:HasValue(self.data.tAlwaysAllowList, strValue)
	
	if nIndex then
		table.remove(self.data.tAlwaysAllowList, nIndex)
	end
	
	--add to filter list unless its already there
	nIndex = self:HasValue(self.data.tFilteredList, strValue)
	
	if not nIndex then
		table.insert(self.data.tFilteredList, strValue)
	end
end

function InviteFilter:AddToAllowList(strValue)
	--remove from filter list if exists
	local nIndex = self:HasValue(self.data.tFilteredList, strValue)
	
	if nIndex then
		table.remove(self.data.tFilteredList, nIndex)
	end

	--add to allow list unless its already there
	nIndex = self:HasValue(self.data.tAlwaysAllowList, strValue)
	
	if not nIndex then
		table.insert(self.data.tAlwaysAllowList, strValue)
	end
end

function InviteFilter:RemoveFromLists(strValue)
	local nFilterIndex = self:HasValue(self.data.tFilteredList, strValue)
	local nAllowIndex = self:HasValue(self.data.tAlwaysAllowList, strValue)
	
	if nFilterIndex then
		table.remove(self.data.tFilteredList, nFilterIndex)
	end
	
	if nAllowIndex then
		table.remove(self.data.tAlwaysAllowList, nAllowIndex)
	end
end

function InviteFilter:QueryLists(strValue)
	local nFilterIndex, nAllowIndex, strOutput;
	
	strOutput = string.format("%s - not found", strValue)
	nFilterIndex = self:HasValue(self.data.tFilteredList, strValue)	
	nAllowIndex = self:HasValue(self.data.tAlwaysAllowList, strValue)
	
	if nFilterIndex then
		strOutput = string.format("%s - filtered", strValue)
	elseif nAllowIndex then
		strOutput = string.format("%s - allowed", strValue)
	end
	
	self:DisplayAddonMessage(strOutput)
end

function InviteFilter:OnGuildInvite( strGuildName, strInvitorName, guildType )
	local strGuildType = self:IsHandlingInviteType(guildType)
	
	if not strGuildType then
		return
	end
	
	if self:IsFriendOrGuildie(strInvitorName) or self:IsAlwaysAllowed(strInvitorName, strGuildName) then
		return
	end
	
	if self:IsOnFilteredList(strInvitorName, strGuildName) then
		GuildLib.Decline()
		self:CloseParentAddonWindow()
	else
		strCurrentInvitor = strInvitorName
		strCurrentGuild = strGuildName
		strCurrentGuildType = strGuildType
		self:ShowInviteFilter()
	end
end

function InviteFilter:IsOnFilteredList(strInvitor, strGuildName)
	if self:FilterContainsInvitor(strInvitor) or self:FilterContainsGuild(strGuildName) then
		return true
	end
	
	return false
end

function InviteFilter:IsFriendOrGuildie(strInvitor)
	local tRelationships = GameLib.SearchRelationshipStatusByCharacterName(strInvitor)
	if tRelationships and (tRelationships.tFriend or tRelationships.tAccountFriend or tRelationships.tGuilds or tRelationships.nGuildIndex) then
		return true
	end
	
	return false
end

function InviteFilter:IsAlwaysAllowed(strInvitor, strGroupName)
	if self:ShouldAlwaysAllowInvitor(strInvitor) or self:ShouldAlwaysAllowGuild(strGroupName) then
		return true
	end
	
	return false
end

function InviteFilter:FilterContainsInvitor(strInvitor)
	local bHasValue = self:HasValue(self.data.tFilteredList, strInvitor)
	return bHasValue
end

function InviteFilter:FilterContainsGuild(strGroupName)
	local bHasValue = self:HasValue(self.data.tFilteredList, strGroupName)
	return bHasValue
end

function InviteFilter:ShouldAlwaysAllowInvitor(strInvitor)
	local bHasValue = self:HasValue(self.data.tAlwaysAllowList, strInvitor)
	return bHasValue
end

function InviteFilter:ShouldAlwaysAllowGuild(strGroupName)
	local bHasValue = self:HasValue(self.data.tAlwaysAllowList, strGroupName)
	return bHasValue
end

function InviteFilter:IsHandlingInviteType(type)	
	if (type == GuildLib.GuildType_ArenaTeam_2v2 or type == GuildLib.GuildType_ArenaTeam_3v3 or type == GuildLib.GuildType_ArenaTeam_5v5) and self.data.bHandleArenaTeams then 
		return "Arena team"
	elseif type == GuildLib.GuildType_Guild and self.data.bHandleGuilds then
		return "Guild"
	elseif type == GuildLib.GuildType_WarParty and self.data.bHandleWarParties then
		return "War Party"
	elseif type == GuildLib.GuildType_Circle and self.data.bHandleCirles then
		return "Circle"
	end
	
	return nil
end
-----------------------------------------------------------------------------------------------
-- InviteFilterForm Functions
-----------------------------------------------------------------------------------------------
-- when the Always allow invitor button is clicked
function InviteFilter:OnAlwaysAllowInvitor()
	self.wndMain:Close() -- hide the window
	self:AddValueAndClear(self.data.tAlwaysAllowList, strCurrentInvitor)
end

function InviteFilter:OnAlwaysAllowGuild()
	self.wndMain:Close() -- hide the window
	self:AddValueAndClear(self.data.tAlwaysAllowList, strCurrentGuild)
end

function InviteFilter:OnFilterInvitor()	
	self.wndMain:Close() -- hide the window
	self:AddValueAndClear(self.data.tFilteredList, strCurrentInvitor)
end

function InviteFilter:OnFilterGuild()	
	self.wndMain:Close() -- hide the window
	self:AddValueAndClear(self.data.tFilteredList, strCurrentGuild)
end

-- when the Cancel button is clicked
function InviteFilter:OnCancel()
	self.wndMain:Destroy()
end

function InviteFilter:OnCloseOptions()
	if self.wndOptions and self.wndOptions:IsValid() then
		self.wndOptions:Destroy()
	end
end

function InviteFilter:ShowInviteFilter()
	if strCurrentGuild == nil or strCurrentInvitor == nil or strCurrentGuildType == nil then
		return
	end
	
	self.wndMain:FindChild("FilterInviteTitle"):SetText(strCurrentGuildType .. ": " .. strCurrentGuild .. " from " .. strCurrentInvitor)
	self.wndMain:FindChild("FilterGuildBtn"):SetText("Guild: " .. strCurrentGuild)
	self.wndMain:FindChild("FilterInvitorBtn"):SetText("Invitor: " .. strCurrentInvitor)
	self.wndMain:Show(true)
end

function InviteFilter:InitAndShowOptions()
	if self.wndOptions == nil or not self.wndOptions:IsValid() then
		self.wndOptions = Apollo.LoadForm(self.xmlDoc, "InviteFilterOptionsForm", nil, self)
	end
	
	if not self.wndOptions or not self.wndOptions:IsValid() then
		self:DisplayAddonMessage("Error opening options window")
		return
	end
	
	self:ShowFilterOptions()
end

function InviteFilter:ShowFilterOptions()
	self.wndOptions:FindChild("GuildInvitesBtn"):SetCheck(self.data.bHandleGuilds)
	self.wndOptions:FindChild("ArenaTeamsInvitesBtn"):SetCheck(self.data.bHandleArenaTeams)
	self.wndOptions:FindChild("CircleInvitesBtn"):SetCheck(self.data.bHandleCirles)
	self.wndOptions:FindChild("WarPartyInvitesBtn"):SetCheck(self.data.bHandleWarParties)
	self.wndOptions:Show(true)
	self.wndOptions:ToFront()
end

function InviteFilter:CloseParentAddonWindow()
	local wndParent = self.arenaTeamAddon[kstrWndArenaTeamInvite]
	
	if wndParent then
		wndParent:Destroy()
	end
end
---------------------------------------------------------------------------------------------------
-- InviteFilterOptionsForm Functions
---------------------------------------------------------------------------------------------------

function InviteFilter:OnGuildInvitesHandled( wndHandler, wndControl, eMouseButton )
	self.data.bHandleGuilds = true
end

function InviteFilter:OnGuildInvitesUnhandled( wndHandler, wndControl, eMouseButton )
	self.data.bHandleGuilds = false
end

function InviteFilter:ArenaTeamInvitesHandled( wndHandler, wndControl, eMouseButton )
	self.data.bHandleArenaTeams = true
end

function InviteFilter:ArenaTeamInvitesUnhandled( wndHandler, wndControl, eMouseButton )
	self.data.bHandleArenaTeams = false
end

function InviteFilter:CircleInvitesHandled( wndHandler, wndControl, eMouseButton )
	self.data.bHandleCirles = true
end

function InviteFilter:CircleInvitesUnhandled( wndHandler, wndControl, eMouseButton )
	self.data.bHandleCirles = false
end

function InviteFilter:WarPartyInvitesHandled( wndHandler, wndControl, eMouseButton )
	self.data.bHandleWarParties = true
end

function InviteFilter:WarPartyInvitesUnhandled( wndHandler, wndControl, eMouseButton )
	self.data.bHandleWarParties = false
end

-----------------------------------------------------------------------------------------------
-- InviteFilter Helper functions
-----------------------------------------------------------------------------------------------
function InviteFilter:DisplayAddonMessage(strMessage)
	local strOutput = string.format("InviteFilter: %s", strMessage)
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, strOutput, "")
end

function InviteFilter:ClearCurrentValues()
	strCurrentInvitor = nil
	strCurrentGuild = nil
	strCurrentGuildType = nil
end


function InviteFilter:AddValueAndClear(tTable, strValue)
	table.insert(tTable, strValue)
	self:ClearCurrentValues()
end

function InviteFilter:HasValue(tTable, strValue)
	for i, value in pairs(tTable) do
		if value:lower() == strValue:lower() then
			return i
		end
	end
	
	return nil
end

function InviteFilter:SetValueOrDefault(loadedValue, defaultValue)
	if loadedValue ~= nil then
		return loadedValue
	end
	
	return defaultValue
end

-----------------------------------------------------------------------------------------------
-- InviteFilter Instance
-----------------------------------------------------------------------------------------------
local InviteFilterInst = InviteFilter:new()
InviteFilterInst:Init()
